import pytest
import numpy as np
import astropy.units as u
import named_arrays as na
from ..tests import test_vectors
from ..cartesian.tests import test_vectors_cartesian

_num_x = test_vectors_cartesian._num_x
_num_y = test_vectors_cartesian._num_y
_num_z = test_vectors_cartesian._num_z
_num_distribution = test_vectors_cartesian._num_distribution


def _temporal_spectral_directional_arrays() -> (
    list[na.TemporalSpectralDirectionalVectorArray]
):
    return [
        na.TemporalSpectralDirectionalVectorArray(
            time=10 * u.s,
            wavelength=500 * u.nm,
            direction=na.Cartesian2dVectorArray(1, 2) * u.mm,
        ),
        na.TemporalSpectralDirectionalVectorArray(
            time=na.linspace(0, 10, axis="y", num=_num_y) * u.s,
            wavelength=na.linspace(400, 600, axis="y", num=_num_y) * u.nm,
            direction=na.Cartesian2dVectorLinearSpace(
                1, 2, axis="y", num=_num_y
            ).explicit
            * u.mm,
        ),
    ]


def _temporal_spectral_directional_arrays_2() -> (
    list[na.TemporalSpectralDirectionalVectorArray]
):
    return [
        na.TemporalSpectralDirectionalVectorArray(
            time=20 * u.s,
            wavelength=400 * u.nm,
            direction=na.Cartesian2dVectorArray(3, 4) * u.m,
        ),
        na.TemporalSpectralDirectionalVectorArray(
            time=na.NormalUncertainScalarArray(10 * u.s, width=1 * u.s),
            wavelength=na.NormalUncertainScalarArray(400 * u.nm, width=1 * u.nm),
            direction=na.Cartesian2dVectorArray(
                x=na.NormalUncertainScalarArray(3, width=1) * u.m,
                y=na.NormalUncertainScalarArray(4, width=1) * u.m,
            ),
        ),
    ]


def _temporal_spectral_directional_items() -> (
    list[na.AbstractArray | dict[str, int | slice | na.AbstractArray]]
):
    return [
        dict(y=0),
        dict(y=slice(0, 1)),
        dict(y=na.ScalarArrayRange(0, 2, axis="y")),
    ]


class AbstractTestAbstractTemporalSpectralDirectionalVectorArray(
    test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray,
):

    @pytest.mark.parametrize(
        argnames="item", argvalues=_temporal_spectral_directional_items()
    )
    def test__getitem__(
        self,
        array: na.AbstractSpectralVectorArray,
        item: dict[str, int | slice | na.AbstractArray] | na.AbstractArray,
    ):
        super().test__getitem__(array=array, item=item)

    @pytest.mark.parametrize("array_2", _temporal_spectral_directional_arrays_2())
    class TestUfuncBinary(
        test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestUfuncBinary
    ):
        pass

    @pytest.mark.parametrize("array_2", _temporal_spectral_directional_arrays_2())
    class TestMatmul(
        test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestMatmul,
    ):
        pass

    class TestArrayFunctions(
        test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestArrayFunctions,
    ):

        @pytest.mark.parametrize("array_2", _temporal_spectral_directional_arrays_2())
        class TestAsArrayLikeFunctions(
            test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestArrayFunctions.TestAsArrayLikeFunctions,
        ):
            pass

        @pytest.mark.parametrize(
            argnames="where",
            argvalues=[
                np._NoValue,
                True,
                na.ScalarArray(True),
            ],
        )
        class TestReductionFunctions(
            test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestArrayFunctions.TestReductionFunctions,
        ):
            pass

        @pytest.mark.parametrize(
            argnames="q",
            argvalues=[
                0.25,
                25 * u.percent,
                na.ScalarLinearSpace(0.25, 0.75, axis="q", num=3, endpoint=True),
            ],
        )
        class TestPercentileLikeFunctions(
            test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestArrayFunctions.TestPercentileLikeFunctions,
        ):
            pass

    class TestNamedArrayFunctions(
        test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestNamedArrayFunctions,
    ):
        @pytest.mark.skip
        class TestPltPlotLikeFunctions(
            test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestNamedArrayFunctions.TestPltPlotLikeFunctions,
        ):
            pass


@pytest.mark.parametrize("array", _temporal_spectral_directional_arrays())
class TestTemporalSpectralDirectionalVectorArray(
    AbstractTestAbstractTemporalSpectralDirectionalVectorArray,
    test_vectors_cartesian.AbstractTestAbstractExplicitCartesianVectorArray,
):
    @pytest.mark.parametrize(
        argnames="item",
        argvalues=[
            dict(y=0),
            dict(y=slice(None)),
        ],
    )
    @pytest.mark.parametrize(
        argnames="value",
        argvalues=[
            700 * u.nm,
        ],
    )
    def test__setitem__(
        self,
        array: na.ScalarArray,
        item: dict[str, int | slice | na.ScalarArray] | na.ScalarArray,
        value: float | na.ScalarArray,
    ):
        super().test__setitem__(array=array, item=item, value=value)


class AbstractTestAbstractImplicitTemporalSpectralDirectionalVectorArray(
    AbstractTestAbstractTemporalSpectralDirectionalVectorArray,
    test_vectors_cartesian.AbstractTestAbstractImplicitCartesianVectorArray,
):
    pass


class AbstractTestAbstractParameterizedTemporalSpectralDirectionalVectorArray(
    AbstractTestAbstractImplicitTemporalSpectralDirectionalVectorArray,
    test_vectors_cartesian.AbstractTestAbstractParameterizedCartesianVectorArray,
):
    pass


class AbstractTestAbstractTemporalSpectralDirectionalVectorSpace(
    AbstractTestAbstractParameterizedTemporalSpectralDirectionalVectorArray,
    test_vectors_cartesian.AbstractTestAbstractCartesianVectorSpace,
):
    pass


def _temporal_spectral_directional_linear_spaces() -> (
    list[na.TemporalSpectralDirectionalVectorLinearSpace]
):
    return [
        na.TemporalSpectralDirectionalVectorLinearSpace(
            start=400 * u.nm,
            stop=600 * u.nm,
            axis="y",
            num=_num_y,
        )
    ]


@pytest.mark.parametrize("array", _temporal_spectral_directional_linear_spaces())
class TestTemporalSpectralDirectionalVectorLinearSpace(
    AbstractTestAbstractTemporalSpectralDirectionalVectorSpace,
    test_vectors_cartesian.AbstractTestAbstractCartesianVectorLinearSpace,
):
    pass


@pytest.mark.parametrize(
    argnames="array",
    argvalues=[
        na.ExplicitTemporalWcsSpectralDirectionalVectorArray(
            time=10 * u.s,
            crval=na.SpectralDirectionalVectorArray(
                wavelength=500 * u.nm,
                direction=na.Cartesian2dVectorArray(1, 1) * u.deg,
            ),
            crpix=na.CartesianNdVectorArray(
                dict(
                    wavelength=1,
                    x=2,
                    y=3,
                )
            ),
            cdelt=na.SpectralDirectionalVectorArray(
                wavelength=1 * u.nm,
                direction=na.Cartesian2dVectorArray(1, 1) * u.arcsec,
            ),
            pc=na.SpectralDirectionalMatrixArray(
                wavelength=na.CartesianNdVectorArray(dict(wavelength=1, x=0, y=0)),
                direction=na.Cartesian2dMatrixArray(
                    x=na.CartesianNdVectorArray(dict(wavelength=0, x=1, y=0)),
                    y=na.CartesianNdVectorArray(dict(wavelength=0, x=0, y=1)),
                ),
            ),
            shape_wcs=dict(wavelength=5, x=_num_x, y=_num_y),
        ),
    ],
)
class TestExplicitTemporalWcsSpectralDirectionalVectorArray(
    AbstractTestAbstractImplicitTemporalSpectralDirectionalVectorArray,
    test_vectors.AbstractTestAbstractWcsVector,
):
    pass



from typing import Type, Callable, Sequence
import pytest
import numpy as np
import astropy.units as u
import named_arrays as na
import named_arrays.tests.test_core
from . import test_vectors_cartesian

__all__ = [
    'AbstractTestAbstractCartesianNdVectorArray',
    'TestCartesianNdVectorArray',
]

_num_x = test_vectors_cartesian._num_x
_num_y = test_vectors_cartesian._num_y
_num_z = test_vectors_cartesian._num_z
_num_distribution = test_vectors_cartesian._num_distribution


def _cartesian_nd_arrays():
    units = [1, u.mm]
    arrays_numeric_x = [
        4,
        na.ScalarUniformRandomSample(-4, 4, shape_random=dict(y=_num_y)),
    ]

    arrays_numeric_y = [
        5.,
        na.UniformUncertainScalarArray(
            nominal=na.ScalarUniformRandomSample(-5, 5, shape_random=dict(x=_num_x, y=_num_y)),
            width=1,
            num_distribution=_num_distribution
        ),
        na.Cartesian2dVectorArray(x=7.3, y=5)
    ]

    arrays = [
        na.CartesianNdVectorArray(components=dict(x=ax, y=ay)) * unit
        for ax in arrays_numeric_x
        for ay in arrays_numeric_y
        for unit in units
    ]

    return arrays


def _cartesian_nd_arrays_2():
    units = [1, u.mm]
    arrays_scalar = [
        6,
        na.ScalarArray(6),
        na.ScalarUniformRandomSample(-6, 6, shape_random=dict(y=_num_y, x=_num_x)),
        na.UniformUncertainScalarArray(
            nominal=na.ScalarUniformRandomSample(-6, 6, shape_random=dict(y=_num_y, x=_num_x)),
            width=1,
            num_distribution=_num_distribution
        )
    ]
    arrays_scalar = [a * unit for a in arrays_scalar for unit in units]

    arrays_vector_x = [
        7,
        na.ScalarUniformRandomSample(-7, 7, shape_random=dict(y=_num_y)),
    ]
    arrays_vector_y = [
        8,
        na.ScalarUniformRandomSample(-8, 8, shape_random=dict(y=_num_y, x=_num_x)),
    ]
    arrays_vector = [
        na.CartesianNdVectorArray(dict(x=ax, y=ay))  * unit
        for ax in arrays_vector_x
        for ay in arrays_vector_y
        for unit in units
    ]

    arrays = arrays_scalar + arrays_vector
    return arrays


def _cartesian_nd_items() -> list[na.AbstractArray | dict[str, int, slice, na.AbstractArray]]:
    return [
        dict(y=0),
        dict(y=slice(0, 1)),
        dict(y=na.ScalarArrayRange(0, 2, axis='y')),
        dict(
            y=na.CartesianNdVectorArray(dict(
                x=na.ScalarArrayRange(0, 2, axis='y'),
                y=na.ScalarArrayRange(0, 2, axis='y'),
            ))
        ),
        dict(
            y=na.UncertainScalarArray(
                nominal=na.ScalarArray(np.array([0, 1]), axes=('y',)),
                distribution=na.ScalarArray(
                    ndarray=np.array([[0, ], [1, ]]),
                    axes=('y', na.UncertainScalarArray.axis_distribution),
                )
            ),
            _distribution=na.UncertainScalarArray(
                nominal=None,
                distribution=na.ScalarArray(
                    ndarray=np.array([[0], [0]]),
                    axes=('y', na.UncertainScalarArray.axis_distribution),
                )
            )
        ),
        na.ScalarLinearSpace(0, 1, axis='y', num=_num_y) > 0.5,
        na.UniformUncertainScalarArray(
            nominal=na.ScalarLinearSpace(0, 1, axis='y', num=_num_y),
            width=0.1,
            num_distribution=_num_distribution,
        ) > 0.5,
        na.CartesianNdVectorArray(dict(
            x=na.ScalarLinearSpace(0, 1, axis='y', num=_num_y) > 0.3,
            y=na.ScalarLinearSpace(0, 1, axis='y', num=_num_y) > 0.5,
        )),
    ]


class AbstractTestAbstractCartesianNdVectorArray(
    test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray,
):

    @pytest.mark.parametrize(
        argnames='item',
        argvalues=_cartesian_nd_items()
    )
    def test__getitem__(
            self,
            array: na.AbstractCartesianNdVectorArray,
            item: dict[str, int | slice | na.AbstractArray] | na.AbstractArray,
    ):
        super().test__getitem__(array=array, item=item)

    @pytest.mark.parametrize('array_2', _cartesian_nd_arrays_2())
    class TestUfuncBinary(
        test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestUfuncBinary
    ):
        pass

    @pytest.mark.parametrize('array_2', _cartesian_nd_arrays_2())
    class TestMatmul(
        test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestMatmul
    ):
        pass

    class TestArrayFunctions(
        test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestArrayFunctions
    ):
        @pytest.mark.parametrize("array_2", _cartesian_nd_arrays_2())
        class TestAsArrayLikeFunctions(
            test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestArrayFunctions.TestAsArrayLikeFunctions,
        ):
            pass

        @pytest.mark.parametrize(
            argnames='where',
            argvalues=[
                np._NoValue,
                (na.ScalarLinearSpace(-1, 1, 'x', _num_x) >= 0) | (na.ScalarLinearSpace(-1, 1, 'y', _num_y) >= 0),
                na.CartesianNdVectorArray(dict(
                    x=(na.ScalarLinearSpace(-1, 1, 'x', _num_x) >= 0) | (na.ScalarLinearSpace(-1, 1, 'y', _num_y) >= 0),
                    y=(na.ScalarLinearSpace(-1, 1, 'x', _num_x) <= 0) | (na.ScalarLinearSpace(-1, 1, 'y', _num_y) <= 0),
                ))
            ]
        )
        class TestReductionFunctions(
            test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestArrayFunctions.TestReductionFunctions,
        ):
            pass

        @pytest.mark.parametrize(
            argnames='q',
            argvalues=[
                25 * u.percent,
                na.ScalarLinearSpace(.25, .75, axis='q', num=3, endpoint=True),
                na.CartesianNdVectorArray(dict(x=25 * u.percent, y=35 * u.percent)),
                na.CartesianNdVectorArray(dict(
                    x=na.ScalarLinearSpace(.25, .50, axis='q', num=3, endpoint=True),
                    y=na.ScalarLinearSpace(50 * u.percent, 75 * u.percent, axis='q', num=3, endpoint=True)
                ))
            ]
        )
        class TestPercentileLikeFunctions(
            test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestArrayFunctions.
            TestPercentileLikeFunctions,
        ):
            pass

    class TestNamedArrayFunctions(
        test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestNamedArrayFunctions,
    ):
        @pytest.mark.xfail(raises=TypeError)
        class TestPltPlotLikeFunctions(
            test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestNamedArrayFunctions.TestPltPlotLikeFunctions,
        ):
            pass


@pytest.mark.parametrize('array', _cartesian_nd_arrays())
class TestCartesianNdVectorArray(
    AbstractTestAbstractCartesianNdVectorArray,
    test_vectors_cartesian.AbstractTestAbstractExplicitCartesianVectorArray,
):

    @pytest.mark.parametrize(
        argnames="item",
        argvalues=[
            dict(y=0),
            dict(x=0, y=0),
            dict(y=slice(None)),
            dict(y=na.ScalarArrayRange(0, _num_y, axis='y')),
            dict(x=na.ScalarArrayRange(0, _num_x, axis='x'), y=na.ScalarArrayRange(0, _num_y, axis='y')),
            dict(
                y=na.CartesianNdVectorArray(dict(
                    x=na.ScalarArrayRange(0, _num_y, axis='y'),
                    y=na.ScalarArrayRange(0, _num_y, axis='y'),
                ))
            ),
            na.ScalarArray.ones(shape=dict(y=_num_y), dtype=bool),
            np.ones_like(na.CartesianNdVectorArray(dict(x=0, y=0)), dtype=bool, shape=dict(y=_num_y)),
        ],
    )
    @pytest.mark.parametrize(
        argnames="value",
        argvalues=[
            0,
            na.ScalarUniformRandomSample(-5, 5, dict(y=_num_y)),
            na.CartesianNdVectorArray(
                dict(
                    x=na.ScalarUniformRandomSample(-5, 5, shape_random=dict(y=_num_y)),
                    y=na.ScalarUniformRandomSample(-5, 5, shape_random=dict(y=_num_y))
                )
            ),
        ]
    )
    def test__setitem__(
            self,
            array: na.ScalarArray,
            item: dict[str, int | slice | na.ScalarArray] | na.ScalarArray,
            value: float | na.ScalarArray
    ):
        super().test__setitem__(array=array, item=item, value=value)


@pytest.mark.parametrize("type_array", [na.CartesianNdVectorArray])
class TestCartesianNdVectorArrayCreation(
    test_vectors_cartesian.AbstractTestAbstractExplicitCartesianVectorArrayCreation,
):
    @pytest.mark.parametrize("like", [None] + _cartesian_nd_arrays())
    class TestFromScalarArray(
        test_vectors_cartesian.AbstractTestAbstractExplicitCartesianVectorArrayCreation.TestFromScalarArray,
    ):
        pass



import pytest
import named_arrays as na

shape = dict(x=101, y=101)

img = na.random.normal(0, 6.5, shape)
spikes = 1000 * na.random.poisson(0.001, shape)


@pytest.mark.parametrize(
    argnames="array",
    argvalues=[
        img + 100 * spikes,
        img + na.NormalUncertainScalarArray(100, 5) * spikes,
        na.FunctionArray(
            inputs=None,
            outputs=img + 100 * spikes,
        ),
    ],
)
@pytest.mark.parametrize(
    argnames="axis",
    argvalues=[
        tuple(shape),
    ],
)
@pytest.mark.parametrize("where", [None])
@pytest.mark.parametrize("inbkg", [None])
@pytest.mark.parametrize("invar", [None])
@pytest.mark.parametrize("psfk", [
    None,
    na.ScalarArray.ones(dict(x=5, y=5)),
])
def test_despike(
    array: na.AbstractArray,
    axis: tuple[str, str],
    where: None | bool | na.AbstractArray,
    inbkg: None | na.AbstractArray,
    invar: None | float | na.AbstractArray,
    psfk: None | na.AbstractArray,
):
    result = na.despike(
        array=array,
        axis=axis,
        where=where,
        inbkg=inbkg,
        invar=invar,
        psfk=psfk,
    )

    assert result.sum() != array.sum()



from __future__ import annotations
from typing import Type, TypeVar
from typing_extensions import Self
import dataclasses
import named_arrays as na

__all__ = [
    'AbstractSpectralPositionalVectorArray',
    'SpectralPositionalVectorArray',
    'AbstractImplicitSpectralPositionalVectorArray',
    'AbstractParameterizedSpectralPositionalVectorArray',
    'AbstractSpectralPositionalVectorSpace',
    'SpectralPositionalVectorLinearSpace',
]

PositionT = TypeVar("PositionT", bound=na.ArrayLike)
WavelengthT = TypeVar("WavelengthT", bound=na.ScalarLike)


@dataclasses.dataclass(eq=False, repr=False)
class AbstractSpectralPositionalVectorArray(
    na.AbstractPositionalVectorArray,
    na.AbstractSpectralVectorArray,
):

    @property
    def type_abstract(self) -> Type[na.AbstractArray]:
        return AbstractSpectralPositionalVectorArray

    @property
    def type_explicit(self) -> Type[na.AbstractExplicitArray]:
        return SpectralPositionalVectorArray

    @property
    def type_matrix(self) -> Type[na.SpectralPositionalMatrixArray]:
        return na.SpectralPositionalMatrixArray


@dataclasses.dataclass(eq=False, repr=False)
class SpectralPositionalVectorArray(
    AbstractSpectralPositionalVectorArray,
    na.PositionalVectorArray[PositionT],
    na.SpectralVectorArray[WavelengthT],
):

    @classmethod
    def from_scalar(
            cls: Type[Self],
            scalar: na.AbstractScalar,
            like: None | na.AbstractExplicitVectorArray = None,
    ) -> SpectralPositionalVectorArray:
        return cls(wavelength=scalar, position=scalar)


@dataclasses.dataclass(eq=False, repr=False)
class AbstractImplicitSpectralPositionalVectorArray(
    AbstractSpectralPositionalVectorArray,
    na.AbstractImplicitPositionalVectorArray,
    na.AbstractImplicitSpectralVectorArray,
):
    pass


@dataclasses.dataclass(eq=False, repr=False)
class AbstractParameterizedSpectralPositionalVectorArray(
    AbstractImplicitSpectralPositionalVectorArray,
    na.AbstractParameterizedVectorArray,
):
    pass


@dataclasses.dataclass(eq=False, repr=False)
class AbstractSpectralPositionalVectorSpace(
    AbstractParameterizedSpectralPositionalVectorArray,
    na.AbstractVectorSpace,
):
    pass


@dataclasses.dataclass(eq=False, repr=False)
class SpectralPositionalVectorLinearSpace(
    AbstractSpectralPositionalVectorSpace,
    na.AbstractVectorLinearSpace,
):
    pass



"""
A thin wrapper around the :mod:`ndfilters` package for named arrays.
"""

from __future__ import annotations
from typing import TypeVar, Literal
import named_arrays as na

__all__ = [
    "mean_filter",
    "trimmed_mean_filter",
    "variance_filter",
]

ArrayT = TypeVar("ArrayT", bound="na.AbstractArray")
WhereT = TypeVar("WhereT", bound="bool | na.AbstractArray")


def mean_filter(
    array: ArrayT,
    size: dict[str, int],
    where: WhereT = True,
) -> ArrayT | WhereT:
    """
    A thin wrapper around :func:`ndfilters.mean_filter` for named arrays.

    Parameters
    ----------
    array
        The input array to be filtered.
    size
        The shape of the kernel over which the mean will be calculated.
    where
        A boolean mask used to select which elements of the input array are to
        be filtered.

    Examples
    --------

    Filter a sample image.

    .. jupyter-execute::
        :stderr:

        import matplotlib.pyplot as plt
        import scipy.datasets
        import named_arrays as na

        img = na.ScalarArray(scipy.datasets.ascent(), axes=("y", "x"))

        img_filtered = na.ndfilters.mean_filter(img, size=dict(x=21, y=21))

        fig, axs = plt.subplots(
            ncols=2,
            sharex=True,
            sharey=True,
            constrained_layout=True,
        )
        axs[0].set_title("original image");
        na.plt.imshow(
            X=img,
            axis_x="x",
            axis_y="y",
            ax=axs[0],
            cmap="gray",
        );
        axs[1].set_title("filtered image");
        na.plt.imshow(
            X=img_filtered,
            axis_x="x",
            axis_y="y",
            ax=axs[1],
            cmap="gray",
        );
    """
    return na._named_array_function(
        func=mean_filter,
        array=array,
        size=size,
        where=where,
    )


def trimmed_mean_filter(
    array: ArrayT,
    size: dict[str, int],
    where: WhereT = True,
    mode: Literal["mirror", "nearest", "wrap", "truncate"] = "mirror",
    proportion: float = 0.25,
) -> ArrayT | WhereT:
    """
    A thin wrapper around :func:`ndfilters.trimmed_mean_filter` for named arrays.

    Parameters
    ----------
    array
        The input array to be filtered.
    size
        The shape of the kernel over which the mean will be calculated.
    where
        A boolean mask used to select which elements of the input array are to
        be filtered.
    mode
        The method used to extend the input array beyond its boundaries.
        See :func:`scipy.ndimage.generic_filter` for the definitions.
        Currently, only "mirror", "nearest", "wrap", and "truncate" modes are
        supported.
    proportion
        The proportion to cut from the top and bottom of the distribution.

    Examples
    --------

    Filter a sample image.

    .. jupyter-execute::
        :stderr:

        import matplotlib.pyplot as plt
        import scipy.datasets
        import named_arrays as na

        img = na.ScalarArray(scipy.datasets.ascent(), axes=("y", "x"))

        img_filtered = na.ndfilters.trimmed_mean_filter(
            array=img,
            size=dict(x=21, y=21),
        )

        fig, axs = plt.subplots(
            ncols=2,
            sharex=True,
            sharey=True,
            constrained_layout=True,
        )
        axs[0].set_title("original image");
        na.plt.imshow(
            X=img,
            axis_x="x",
            axis_y="y",
            ax=axs[0],
            cmap="gray",
        );
        axs[1].set_title("filtered image");
        na.plt.imshow(
            X=img_filtered,
            axis_x="x",
            axis_y="y",
            ax=axs[1],
            cmap="gray",
        );
    """
    return na._named_array_function(
        func=trimmed_mean_filter,
        array=array,
        size=size,
        where=where,
        mode=mode,
        proportion=proportion,
    )


def variance_filter(
    array: ArrayT,
    size: dict[str, int],
    where: WhereT = True,
) -> ArrayT | WhereT:
    """
    A thin wrapper around :func:`ndfilters.variance_filter` for named arrays.

    Parameters
    ----------
    array
        The input array to be filtered.
    size
        The shape of the kernel over which the variance will be calculated.
    where
        A boolean mask used to select which elements of the input array are to
        be filtered.

    Examples
    --------

    Filter a sample image.

    .. jupyter-execute::
        :stderr:

        import matplotlib.pyplot as plt
        import scipy.datasets
        import named_arrays as na

        img = na.ScalarArray(scipy.datasets.ascent(), axes=("y", "x"))

        img_filtered = na.ndfilters.variance_filter(img, size=dict(x=21, y=21))

        fig, axs = plt.subplots(
            ncols=2,
            sharex=True,
            sharey=True,
            constrained_layout=True,
        )
        axs[0].set_title("original image");
        na.plt.imshow(
            X=img,
            axis_x="x",
            axis_y="y",
            ax=axs[0],
            cmap="gray",
        );
        axs[1].set_title("filtered image");
        na.plt.imshow(
            X=img_filtered,
            axis_x="x",
            axis_y="y",
            ax=axs[1],
            cmap="gray",
        );
    """
    return na._named_array_function(
        func=variance_filter,
        array=array,
        size=size,
        where=where,
    )



import pytest
import numpy as np
import astropy.units as u
import named_arrays as na
from . import test_vectors
from ..cartesian.tests import test_vectors_cartesian

_num_x = test_vectors._num_x
_num_y = test_vectors._num_y
_num_z = test_vectors._num_z
_num_distribution = test_vectors._num_distribution


def _input_output_arrays() -> list[na.InputOutputVectorArray]:
    return [
        na.InputOutputVectorArray(
            input=0,
            output=1,
        ),
        na.InputOutputVectorArray(
            input=na.linspace(0, 1, axis="y", num=_num_y),
            output=2,
        ),
    ]


def _input_output_arrays_2() -> list[na.InputOutputVectorArray]:
    return [
        na.InputOutputVectorArray(
            input=3,
            output=4,
        ),
        na.InputOutputVectorArray(
            input=na.NormalUncertainScalarArray(100, width=1),
            output=5,
        )
    ]


def _input_output_items() -> list[na.AbstractInputOutputVectorArray | dict[str, int | slice | na.AbstractInputOutputVectorArray]]:
    return [
        dict(y=0),
        dict(y=slice(0, 1)),
        dict(y=na.ScalarArrayRange(0, 2, axis='y')),
    ]


class AbstractTestAbstractInputOutputVectorArray(
    test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray,
):
    def test_input(self, array: na.AbstractInputOutputVectorArray):
        assert isinstance(na.as_named_array(array.input), na.AbstractArray)

    def test_output(self, array: na.AbstractInputOutputVectorArray):
        assert isinstance(na.as_named_array(array.output), na.AbstractArray)

    @pytest.mark.parametrize(
        argnames='item',
        argvalues=_input_output_items()
    )
    def test__getitem__(
            self,
            array: na.AbstractInputOutputVectorArray,
            item: dict[str, int | slice | na.AbstractArray] | na.AbstractArray
    ):
        super().test__getitem__(array=array, item=item)

    @pytest.mark.parametrize('array_2', _input_output_arrays_2())
    class TestUfuncBinary(
        test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestUfuncBinary
    ):
        pass

    @pytest.mark.parametrize('array_2', _input_output_arrays_2())
    class TestMatmul(
        test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestMatmul
    ):
        pass

    class TestArrayFunctions(
        test_vectors.AbstractTestAbstractVectorArray.TestArrayFunctions
    ):

        @pytest.mark.parametrize("array_2", _input_output_arrays_2())
        class TestAsArrayLikeFunctions(
            test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestArrayFunctions.TestAsArrayLikeFunctions
        ):
            pass

        @pytest.mark.parametrize(
            argnames='where',
            argvalues=[
                np._NoValue,
                True,
                na.ScalarArray(True),
            ]
        )
        class TestReductionFunctions(
            test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestArrayFunctions.TestReductionFunctions,
        ):
            pass

        @pytest.mark.parametrize(
            argnames='q',
            argvalues=[
                .25,
                25 * u.percent,
                na.ScalarLinearSpace(.25, .75, axis='q', num=3, endpoint=True),
            ]
        )
        class TestPercentileLikeFunctions(
            test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestArrayFunctions
            .TestPercentileLikeFunctions,
        ):
            pass

    class TestNamedArrayFunctions(
        test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestNamedArrayFunctions,
    ):
        @pytest.mark.skip
        class TestPltPlotLikeFunctions(
            test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestNamedArrayFunctions.TestPltPlotLikeFunctions,
        ):
            pass


@pytest.mark.parametrize("array", _input_output_arrays())
class TestInputOutputVectorArray(
    AbstractTestAbstractInputOutputVectorArray,
    test_vectors_cartesian.AbstractTestAbstractExplicitCartesianVectorArray,
):
    @pytest.mark.parametrize(
        argnames="item",
        argvalues=[
            dict(y=0),
            dict(y=slice(None)),
        ],
    )
    @pytest.mark.parametrize(
        argnames="value",
        argvalues=[
            700,
        ]
    )
    def test__setitem__(
            self,
            array: na.ScalarArray,
            item: dict[str, int | slice | na.ScalarArray] | na.ScalarArray,
            value: float | na.ScalarArray
    ):
        super().test__setitem__(array=array, item=item, value=value)


class AbstractTestAbstractImplicitInputOutputVectorArray(
    AbstractTestAbstractInputOutputVectorArray,
    test_vectors_cartesian.AbstractTestAbstractImplicitCartesianVectorArray,
):
    pass


class AbstractTestAbstractParameterizedInputOutputVectorArray(
    AbstractTestAbstractImplicitInputOutputVectorArray,
    test_vectors_cartesian.AbstractTestAbstractParameterizedCartesianVectorArray,
):
    pass


class AbstractTestAbstractInputOutputVectorSpace(
    AbstractTestAbstractParameterizedInputOutputVectorArray,
    test_vectors_cartesian.AbstractTestAbstractParameterizedCartesianVectorArray,
):
    pass


def _input_output_linear_spaces() -> list[na.InputOutputVectorLinearSpace]:
    return [
        na.InputOutputVectorLinearSpace(400 * u.nm, 600 * u.nm, axis="y", num=_num_y)
    ]


@pytest.mark.parametrize("array", _input_output_linear_spaces())
class TestInputOutputVectorLinearSpace(
    AbstractTestAbstractInputOutputVectorSpace,
    test_vectors_cartesian.AbstractTestAbstractCartesianVectorLinearSpace,
):
    pass



import pytest
import numpy as np
import matplotlib.axes
import matplotlib.animation
import astropy.units as u
import named_arrays as na

_num_t = 11
_num_w = 12
_num_x = 13
_num_y = 14


@pytest.mark.parametrize(
    argnames="W",
    argvalues=[
        na.linspace(-1, 1, axis="w", num=_num_w) * u.mm,
    ],
)
@pytest.mark.parametrize(
    argnames="X",
    argvalues=[
        na.linspace(-2, 2, axis="x", num=_num_x),
    ],
)
@pytest.mark.parametrize(
    argnames="Y",
    argvalues=[
        na.linspace(-1, 1, axis="y", num=_num_y),
    ],
)
@pytest.mark.parametrize(
    argnames="C",
    argvalues=[
        na.random.uniform(-1, 1, shape_random=dict(w=_num_w, x=_num_x, y=_num_y)),
    ],
)
def test_rgbmesh(
    W: na.AbstractScalar,
    X: na.AbstractScalar,
    Y: na.AbstractScalar,
    C: na.AbstractScalar,
):
    result_1 = na.plt.rgbmesh(
        W,
        X,
        Y,
        C=C,
        axis_wavelength="w",
    )
    result_2 = na.plt.rgbmesh(
        W,
        na.Cartesian2dVectorArray(X, Y),
        C=C,
        axis_wavelength="w",
    )
    result_3 = na.plt.rgbmesh(
        na.SpectralPositionalVectorArray(
            wavelength=W,
            position=na.Cartesian2dVectorArray(X, Y),
        ),
        C=C,
        axis_wavelength="w",
    )
    result_4 = na.plt.rgbmesh(
        C=na.FunctionArray(
            inputs=na.SpectralPositionalVectorArray(
                wavelength=W,
                position=na.Cartesian2dVectorArray(X, Y),
            ),
            outputs=C,
        ),
        axis_wavelength="w",
    )

    assert np.all(result_1 == result_2)
    assert np.all(result_1 == result_3)
    assert np.all(result_1 == result_4)


@pytest.mark.parametrize(
    argnames="TXY",
    argvalues=[
        (
            na.linspace(-1, 1, axis="t", num=_num_t),
            na.linspace(-2, 2, axis="x", num=_num_x),
            na.linspace(-1, 1, axis="y", num=_num_y),
        ),
    ],
)
@pytest.mark.parametrize(
    argnames="C",
    argvalues=[
        na.random.uniform(-1, 1, shape_random=dict(t=_num_t, x=_num_x, y=_num_y)),
    ],
)
@pytest.mark.parametrize(
    argnames="axis_time",
    argvalues=["t"],
)
def test_pcolormovie(
    TXY: tuple[
        na.AbstractScalarArray,
        na.AbstractScalarArray,
        na.AbstractScalarArray,
    ],
    C: na.AbstractScalarArray,
    axis_time: str,
):
    result = na.plt.pcolormovie(
        *TXY,
        C=C,
        axis_time=axis_time,
    )
    assert isinstance(result, matplotlib.animation.FuncAnimation)
    assert isinstance(result.to_jshtml(), str)


@pytest.mark.parametrize(
    argnames="T",
    argvalues=[
        na.linspace(-1, 1, axis="t", num=_num_t) * u.s,
    ],
)
@pytest.mark.parametrize(
    argnames="W",
    argvalues=[
        na.linspace(-1, 1, axis="w", num=_num_w) * u.mm,
    ],
)
@pytest.mark.parametrize(
    argnames="X",
    argvalues=[
        na.linspace(-2, 2, axis="x", num=_num_x),
    ],
)
@pytest.mark.parametrize(
    argnames="Y",
    argvalues=[
        na.linspace(-1, 1, axis="y", num=_num_y),
    ],
)
@pytest.mark.parametrize(
    argnames="C",
    argvalues=[
        na.random.uniform(
            low=-1,
            high=1,
            shape_random=dict(t=_num_t, w=_num_w, x=_num_x, y=_num_y),
        ),
    ],
)
def test_rgbmovie(
    T: na.AbstractScalar,
    W: na.AbstractScalar,
    X: na.AbstractScalar,
    Y: na.AbstractScalar,
    C: na.AbstractScalar,
):
    ani_1, cbar_1 = na.plt.rgbmovie(
        T,
        W,
        X,
        Y,
        C=C,
        axis_time="t",
        axis_wavelength="w",
    )
    ani_2, cbar_2 = na.plt.rgbmovie(
        T,
        na.SpectralPositionalVectorArray(
            wavelength=W,
            position=na.Cartesian2dVectorArray(X, Y),
        ),
        C=C,
        axis_time="t",
        axis_wavelength="w",
    )
    ani_3, cbar_3 = na.plt.rgbmovie(
        T,
        W,
        na.Cartesian2dVectorArray(X, Y),
        C=C,
        axis_time="t",
        axis_wavelength="w",
    )
    ani_4, cbar_4 = na.plt.rgbmovie(
        na.TemporalSpectralPositionalVectorArray(
            time=T,
            wavelength=W,
            position=na.Cartesian2dVectorArray(X, Y),
        ),
        C=C,
        axis_time="t",
        axis_wavelength="w",
    )
    ani_5, cbar_5 = na.plt.rgbmovie(
        C=na.FunctionArray(
            inputs=na.TemporalSpectralPositionalVectorArray(
                time=T,
                wavelength=W,
                position=na.Cartesian2dVectorArray(X, Y),
            ),
            outputs=C,
        ),
        axis_time="t",
        axis_wavelength="w",
    )

    assert isinstance(ani_1, matplotlib.animation.FuncAnimation)
    assert isinstance(ani_2, matplotlib.animation.FuncAnimation)
    assert isinstance(ani_3, matplotlib.animation.FuncAnimation)
    assert isinstance(ani_4, matplotlib.animation.FuncAnimation)
    assert isinstance(ani_5, matplotlib.animation.FuncAnimation)

    assert isinstance(ani_1.to_jshtml(), str)
    assert isinstance(ani_2.to_jshtml(), str)
    assert isinstance(ani_3.to_jshtml(), str)
    assert isinstance(ani_4.to_jshtml(), str)
    assert isinstance(ani_5.to_jshtml(), str)

    assert np.all(cbar_1 == cbar_2)
    assert np.all(cbar_1 == cbar_3)
    assert np.all(cbar_1 == cbar_4)
    assert np.all(cbar_1 == cbar_5)


@pytest.mark.parametrize(
    argnames="xlabel,ax",
    argvalues=[
        ("foo", None),
        ("foo", na.plt.subplots(ncols=3)[1]),
    ]
)
def test_set_xlabel(
    xlabel: str | na.AbstractScalar,
    ax: None | matplotlib.axes.Axes | na.AbstractScalar,
):
    na.plt.set_xlabel(xlabel, ax=ax)
    result = na.plt.get_xlabel(ax)
    assert np.all(result == xlabel)


@pytest.mark.parametrize(
    argnames="ylabel,ax",
    argvalues=[
        ("foo", None),
        ("foo", na.plt.subplots(ncols=3)[1]),
    ]
)
def test_set_ylabel(
    ylabel: str | na.AbstractScalar,
    ax: None | matplotlib.axes.Axes | na.AbstractScalar,
):
    na.plt.set_ylabel(ylabel, ax=ax)
    result = na.plt.get_ylabel(ax)
    assert np.all(result == ylabel)


@pytest.mark.parametrize(
    argnames="label,ax",
    argvalues=[
        ("foo", None),
        ("foo", na.plt.subplots(ncols=3)[1]),
    ]
)
def test_set_title(
    label: str | na.AbstractScalar,
    ax: None | matplotlib.axes.Axes | na.AbstractScalar,
):
    na.plt.set_title(label, ax=ax)
    result = na.plt.get_title(ax)
    assert np.all(result == label)


@pytest.mark.parametrize(
    argnames="value,ax",
    argvalues=[
        ("log", None),
        ("log", na.plt.subplots(ncols=3)[1]),
    ]
)
def test_set_xscale(
    value: str | na.AbstractScalar,
    ax: None | matplotlib.axes.Axes | na.AbstractScalar,
):
    na.plt.set_xscale(value, ax=ax)
    result = na.plt.get_xscale(ax)
    assert np.all(result == value)


@pytest.mark.parametrize(
    argnames="value,ax",
    argvalues=[
        ("log", None),
        ("log", na.plt.subplots(ncols=3)[1]),
    ]
)
def test_set_yscale(
    value: str | na.AbstractScalar,
    ax: None | matplotlib.axes.Axes | na.AbstractScalar,
):
    na.plt.set_yscale(value, ax=ax)
    result = na.plt.get_yscale(ax)
    assert np.all(result == value)


@pytest.mark.parametrize(
    argnames="aspect,ax",
    argvalues=[
        (1, None),
        (1, na.plt.subplots(ncols=3)[1]),
        (2, na.plt.subplots(ncols=3)[1]),
    ]
)
def test_set_aspect(
    aspect: str | na.AbstractScalar,
    ax: None | matplotlib.axes.Axes | na.AbstractScalar,
):
    na.plt.set_aspect(aspect, ax=ax)
    result = na.plt.get_aspect(ax)
    assert np.all(result == aspect)


@pytest.mark.parametrize(
    argnames="ax",
    argvalues=[
        None,
        na.plt.subplots(ncols=3)[1]
    ]
)
def test_transAxes(
    ax: None | matplotlib.axes.Axes | na.AbstractScalar,
):
    result = na.plt.transAxes(ax)
    assert isinstance(result, na.AbstractArray)
    assert result.shape == na.shape(ax)


@pytest.mark.parametrize(
    argnames="ax",
    argvalues=[
        None,
        na.plt.subplots(ncols=3)[1]
    ]
)
def test_transData(
    ax: None | matplotlib.axes.Axes | na.AbstractScalar,
):
    result = na.plt.transData(ax)
    assert isinstance(result, na.AbstractArray)
    assert result.shape == na.shape(ax)


@pytest.mark.parametrize(
    argnames="ax",
    argvalues=[
        None,
        na.plt.subplots(ncols=3)[1]
    ]
)
def test_twinx(
    ax: None | matplotlib.axes.Axes | na.AbstractScalar,
):
    result = na.plt.twinx(ax)
    for r in np.nditer(result.ndarray, flags=("refs_ok",)):
        assert isinstance(r.item(), matplotlib.axes.Axes)


@pytest.mark.parametrize(
    argnames="ax",
    argvalues=[
        None,
        na.plt.subplots(ncols=3)[1]
    ]
)
def test_twiny(
    ax: None | matplotlib.axes.Axes | na.AbstractScalar,
):
    result = na.plt.twiny(ax)
    for r in np.nditer(result.ndarray, flags=("refs_ok",)):
        assert isinstance(r.item(), matplotlib.axes.Axes)


@pytest.mark.parametrize(
    argnames="ax",
    argvalues=[
        None,
        na.plt.subplots(ncols=3)[1]
    ]
)
def test_invert_xaxis(
    ax: None | matplotlib.axes.Axes | na.AbstractScalar,
):
    na.plt.invert_xaxis(ax)


@pytest.mark.parametrize(
    argnames="ax",
    argvalues=[
        None,
        na.plt.subplots(ncols=3)[1]
    ]
)
def test_invert_yaxis(
    ax: None | matplotlib.axes.Axes | na.AbstractScalar,
):
    na.plt.invert_yaxis(ax)



from __future__ import annotations
from typing import TypeVar, Generic, Type
from typing_extensions import Self
import abc
import dataclasses
import numpy as np
import astropy.units as u
import named_arrays as na

__all__ = [
    'AbstractCartesianVectorArray',
    'AbstractExplicitCartesianVectorArray',
    'AbstractImplicitCartesianVectorArray',
    'AbstractCartesianVectorRandomSample',
    'AbstractCartesianVectorUniformRandomSample',
    'AbstractCartesianVectorNormalRandomSample',
    'AbstractParameterizedCartesianVectorArray',
    'AbstractCartesianVectorArrayRange',
    'AbstractCartesianVectorSpace',
    'AbstractCartesianVectorLinearSpace',
    'AbstractCartesianVectorStratifiedRandomSpace',
    'AbstractCartesianVectorLogarithmicSpace',
    'AbstractCartesianVectorGeometricSpace',
]


@dataclasses.dataclass(eq=False, repr=False)
class AbstractCartesianVectorArray(
    na.AbstractVectorArray
):
    @property
    @abc.abstractmethod
    def type_explicit(self: Self) -> Type[AbstractExplicitCartesianVectorArray]:
        pass


    @property
    def length(self: Self) -> na.AbstractScalar:
        result = 0
        entries = self.cartesian_nd.entries
        for e in entries:
            if entries[e] is not None:
                result = result + np.square(np.abs(entries[e]))
        result = np.sqrt(result)
        return result

    @property
    def normalized(self: Self) -> Self:
        """
        Return a normalized copy of this vector, where :attr:`length` is unity.
        """
        return self / self.length

    def __mul__(self: Self, other: na.ArrayLike | u.Unit) -> AbstractExplicitCartesianVectorArray:
        if isinstance(other, u.UnitBase):
            components = self.components
            result = self.prototype_vector
            for c in components:
                result.components[c] = components[c] * other
            return result
        else:
            return super().__mul__(other)

    def __lshift__(self: Self, other: na.ArrayLike | u.UnitBase) -> AbstractExplicitCartesianVectorArray:
        if isinstance(other, u.UnitBase):
            components= self.components
            result = self.prototype_vector
            for c in components:
                result.components[c] = components[c] << other
            return result
        else:
            return super().__lshift__(other)

    def __truediv__(self: Self, other: na.ArrayLike | u.UnitBase) -> AbstractExplicitCartesianVectorArray:
        if isinstance(other, u.UnitBase):
            components = self.components
            result = self.prototype_vector
            for c in components:
                result.components[c] = components[c] / other
            return result
        else:
            return super().__truediv__(other)

    def __array_ufunc__(
            self: Self,
            function: np.ufunc,
            method: str,
            *inputs,
            **kwargs,
    ) -> None | AbstractExplicitCartesianVectorArray | tuple[AbstractExplicitCartesianVectorArray, ...]:

        result = super().__array_ufunc__(function, method, *inputs, **kwargs)
        if result is not NotImplemented:
            return result

        if function is np.matmul:
            return NotImplemented

        components = self.components

        components_inputs = []
        for inp in inputs:
            if isinstance(inp, na.AbstractArray):
                if inp.type_abstract == self.type_abstract:
                    components_inp = inp.components
                elif isinstance(inp, na.AbstractScalar):
                    components_inp = {c: inp for c in components}
                else:
                    return NotImplemented
            else:
                components_inp = {c: inp for c in components}
            components_inputs.append(components_inp)

        if "out" in kwargs:
            out = kwargs.pop("out")
            components_out = dict()
            for c in components:
                components_out[c] = tuple(o.components[c] if o is not None else o for o in out)
                components_out[c] = tuple(o if isinstance(np.ndarray, na.AbstractArray) else None for o in out)
        else:
            out = (None, ) * function.nout
            components_out = {c: (None, ) * function.nout for c in components}

        if "where" in kwargs:
            where = kwargs.pop("where")
            if isinstance(where, na.AbstractArray):
                if where.type_abstract == self.type_abstract:
                    components_where = where.components
                elif isinstance(where, na.AbstractScalar):
                    components_where = {c: where for c in components}
                else:
                    return NotImplemented
            else:
                components_where = {c: where for c in components}
        else:
            components_where = {c: True for c in components}

        components_result = tuple(dict() for _ in range(function.nout))
        for c in components:
            component_result = getattr(function, method)(
                *[inp[c] for inp in components_inputs],
                out=components_out[c],
                where=components_where[c],
                **kwargs,
            )
            if function.nout == 1:
                component_result = (component_result, )
            for i in range(function.nout):
                components_result[i][c] = component_result[i]
        result = list(self.type_explicit.from_components(components_result[i]) for i in range(function.nout))

        for i in range(function.nout):
            if out[i] is not None:
                out[i].components = result[i].components
                result[i] = out[i]

        if function.nout == 1:
            result = result[0]
        else:
            result = tuple(result)
        return result


@dataclasses.dataclass(eq=False, repr=False)
class AbstractExplicitCartesianVectorArray(
    AbstractCartesianVectorArray,
    na.AbstractExplicitVectorArray,
):
    pass


@dataclasses.dataclass(eq=False, repr=False)
class AbstractImplicitCartesianVectorArray(
    AbstractCartesianVectorArray,
    na.AbstractImplicitVectorArray,
):
    pass


@dataclasses.dataclass(eq=False, repr=False)
class AbstractCartesianVectorRandomSample(
    AbstractImplicitCartesianVectorArray,
    na.AbstractVectorRandomSample,
):
    pass


StartT = TypeVar('StartT', bound=float | complex | u.Quantity | na.AbstractScalar | AbstractCartesianVectorArray)
StopT = TypeVar('StopT', bound=float | complex | u.Quantity | na.AbstractScalar | AbstractCartesianVectorArray)


@dataclasses.dataclass(eq=False, repr=False)
class AbstractCartesianVectorUniformRandomSample(
    AbstractCartesianVectorRandomSample,
    na.AbstractVectorUniformRandomSample[StartT, StopT],
):
    pass


CenterT = TypeVar('CenterT', bound=float | complex | u.Quantity | na.AbstractScalar | AbstractCartesianVectorArray)
WidthT = TypeVar('WidthT', bound=float | complex | u.Quantity | na.AbstractScalar | AbstractCartesianVectorArray)


@dataclasses.dataclass(eq=False, repr=False)
class AbstractCartesianVectorNormalRandomSample(
    AbstractCartesianVectorRandomSample,
    na.AbstractVectorNormalRandomSample[CenterT, WidthT],
):
    pass


@dataclasses.dataclass(eq=False, repr=False)
class AbstractParameterizedCartesianVectorArray(
    AbstractImplicitCartesianVectorArray,
    na.AbstractParameterizedVectorArray,
):
    pass


@dataclasses.dataclass(eq=False, repr=False)
class AbstractCartesianVectorArrayRange(
    AbstractParameterizedCartesianVectorArray,
    na.AbstractVectorArrayRange[StartT, StopT],
):
    pass


@dataclasses.dataclass(eq=False, repr=False)
class AbstractCartesianVectorSpace(
    AbstractParameterizedCartesianVectorArray,
    na.AbstractVectorSpace,
):
    pass


@dataclasses.dataclass(eq=False, repr=False)
class AbstractCartesianVectorLinearSpace(
    AbstractCartesianVectorSpace,
    na.AbstractVectorLinearSpace,
):
    pass


@dataclasses.dataclass(eq=False, repr=False)
class AbstractCartesianVectorStratifiedRandomSpace(
    AbstractCartesianVectorLinearSpace,
    na.AbstractVectorStratifiedRandomSpace,
):
    pass


@dataclasses.dataclass(eq=False, repr=False)
class AbstractCartesianVectorLogarithmicSpace(
    AbstractCartesianVectorSpace,
    na.AbstractVectorLogarithmicSpace,
):
    pass


@dataclasses.dataclass(eq=False, repr=False)
class AbstractCartesianVectorGeometricSpace(
    AbstractCartesianVectorSpace,
    na.AbstractVectorGeometricSpace,
):
    pass



from typing import Callable

import numpy as np
import named_arrays as na

__all__ = [
    'HANDLED_FUNCTIONS',
]

HANDLED_FUNCTIONS = dict()


def implements(numpy_function: Callable):
    """Register an ``__array_function__`` implementation for :class:`named_array.AbstractArray` objects."""
    def decorator(func):
        HANDLED_FUNCTIONS[numpy_function] = func
        return func
    return decorator


@implements(np.shape)
def shape(
        a: na.AbstractScalarArray,
) -> dict[str, int]:
    return a.shape



import pytest
import numpy as np
import named_arrays as na

shape_vertices = dict(x=10, y=11)
shape_centers = {a: shape_vertices[a] - 1 for a in shape_vertices}

x = na.linspace(-1, 1, axis="x", num=shape_vertices["x"])
y = na.linspace(-1, 1, axis="y",  num=shape_vertices["y"])
z = na.linspace(-1, 1, axis="z", num=3)

x_new = na.linspace(-1, 1, axis="x_new", num=5)
y_new = na.linspace(-1, 1, axis="y_new", num=6)


@pytest.mark.parametrize(
    argnames="coordinates_input,coordinates_output,values_input,axis_input,axis_output,result_expected",
    argvalues=[
        (
            na.linspace(-1, 1, axis="x_input", num=11),
            na.linspace(-1, 1, axis="x_output", num=11),
            np.square(na.linspace(-1, 1, axis="x_input", num=11)),
            None,
            None,
            np.square(na.linspace(-1, 1, axis="x_output", num=11)),
        ),
        (
            y,
            y_new,
            x + y,
            "y",
            "y_new",
            x + y_new,
        ),
        (
            x,
            x_new,
            x + y,
            ("x",),
            ("x_new",),
            x_new + y,
        ),
        (
            x,
            0.1 * x_new + 0.001 * y_new,
            x,
            ("x",),
            ("x_new",),
            0.1 * x_new + 0.001 * y_new,
        ),
    ],
)
def test_regrid_multilinear_1d(
    coordinates_input: tuple[np.ndarray, ...],
    coordinates_output: tuple[np.ndarray, ...],
    values_input: np.ndarray,
    axis_input: None | int | tuple[int, ...],
    axis_output: None | int | tuple[int, ...],
    result_expected: np.ndarray,
):
    result = na.regridding.regrid(
        coordinates_input=coordinates_input,
        coordinates_output=coordinates_output,
        values_input=values_input,
        axis_input=axis_input,
        axis_output=axis_output,
        method="multilinear",
    )
    assert isinstance(result, na.AbstractArray)
    assert np.issubdtype(result.dtype, float)
    assert np.allclose(result, result_expected)


@pytest.mark.parametrize(
    argnames="coordinates_input, values_input, axis_input, coordinates_output, axis_output",
    argvalues=[
        (
            na.Cartesian2dVectorArray(x, y),
            na.random.normal(0, 1, shape_random=shape_centers),
            None,
            na.Cartesian2dVectorArray(
                x=1.1 * x + 0.01,
                y=1.2 * y + 0.01,
            ),
            None,
        ),
        (
            na.Cartesian2dVectorArray(
                x=x + 0.01 * z,
                y=y + 0.01 * z,
            ),
            na.random.normal(0, 1, shape_random=shape_centers | z.shape),
            ("x", "y"),
            na.Cartesian2dVectorArray(
                x=1.1 * (x + 0.001 * z) + 0.01,
                y=1.2 * (y + 0.01 * z) + 0.001,
            ),
            ("x", "y"),
        ),
    ],
)
def test_regrid_conservative_2d(
    coordinates_input: tuple[np.ndarray, ...],
    coordinates_output: tuple[np.ndarray, ...],
    values_input: np.ndarray,
    axis_input: None | int | tuple[int, ...],
    axis_output: None | int | tuple[int, ...],
):
    result = na.regridding.regrid(
        coordinates_input=coordinates_input,
        coordinates_output=coordinates_output,
        values_input=values_input,
        axis_input=axis_input,
        axis_output=axis_output,
        method="conservative",
    )

    if axis_output is None:
        axis_output = tuple(coordinates_output.shape)
    elif isinstance(axis_output, str):
        axis_output = (axis_output, )

    shape_result = coordinates_output.shape
    shape_result = {
        a: shape_result[a] - 1 if a in axis_output
        else shape_result[a]
        for a in shape_result
    }

    assert np.issubdtype(result.dtype, float)
    assert result.shape == shape_result
    assert np.allclose(result.sum(), values_input.sum())



import pytest
import numpy as np
import astropy.units as u
import named_arrays as na
from . import test_vectors
from ..cartesian.tests import test_vectors_cartesian

_num_x = test_vectors._num_x
_num_y = test_vectors._num_y
_num_z = test_vectors._num_z
_num_distribution = test_vectors._num_distribution


def _temporal_arrays() -> list[na.TemporalVectorArray]:
    return [
        na.TemporalVectorArray(time=1 * u.s),
        na.TemporalVectorArray(time=na.ScalarLinearSpace(1, 2, axis="y", num=_num_y).explicit * u.s),
    ]


def _temporal_arrays_2() -> list[na.TemporalVectorArray]:
    return [
        na.TemporalVectorArray(time=3 * u.s),
        na.TemporalVectorArray(time=na.NormalUncertainScalarArray(3, width=1) * u.s),
    ]


def _temporal_items() -> list[na.TemporalVectorArray | dict[str, int | slice | na.TemporalVectorArray]]:
    return [
        dict(y=0),
        dict(y=slice(0, 1)),
        dict(y=na.ScalarArrayRange(0, 2, axis='y')),
    ]


class AbstractTestAbstractTemporalVectorArray(
    test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray,
):
    def test_time(self, array: na.AbstractTemporalVectorArray):
        assert isinstance(na.as_named_array(array.time), (na.AbstractScalar, na.AbstractVectorArray))

    @pytest.mark.parametrize(
        argnames='item',
        argvalues=_temporal_items(),
    )
    def test__getitem__(
            self,
            array: na.AbstractTemporalVectorArray,
            item: dict[str, int | slice | na.AbstractArray] | na.AbstractArray
    ):
        super().test__getitem__(array=array, item=item)

    @pytest.mark.parametrize('array_2', _temporal_arrays_2())
    class TestUfuncBinary(
        test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestUfuncBinary
    ):
        pass

    @pytest.mark.parametrize('array_2', _temporal_arrays_2())
    class TestMatmul(
        test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestMatmul
    ):
        pass

    class TestArrayFunctions(
        test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestArrayFunctions
    ):

        @pytest.mark.parametrize("array_2", _temporal_arrays_2())
        class TestAsArrayLikeFunctions(
            test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestArrayFunctions.TestAsArrayLikeFunctions
        ):
            pass

        @pytest.mark.parametrize(
            argnames='where',
            argvalues=[
                np._NoValue,
            ]
        )
        class TestReductionFunctions(
            test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestArrayFunctions.TestReductionFunctions,
        ):
            pass

        @pytest.mark.parametrize(
            argnames='q',
            argvalues=[
                25 * u.percent,
            ]
        )
        class TestPercentileLikeFunctions(
            test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestArrayFunctions
            .TestPercentileLikeFunctions,
        ):
            pass

    class TestNamedArrayFunctions(
        test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestNamedArrayFunctions,
    ):
        @pytest.mark.skip
        class TestPltPlotLikeFunctions(
            test_vectors_cartesian.AbstractTestAbstractCartesianVectorArray.TestNamedArrayFunctions.TestPltPlotLikeFunctions,
        ):
            pass


@pytest.mark.parametrize("array", _temporal_arrays())
class TestTemporalVectorArray(
    AbstractTestAbstractTemporalVectorArray,
    test_vectors_cartesian.AbstractTestAbstractExplicitCartesianVectorArray,
):
    @pytest.mark.parametrize(
        argnames="item",
        argvalues=[
            dict(y=0),
            dict(y=slice(None)),
        ],
    )
    @pytest.mark.parametrize(
        argnames="value",
        argvalues=[
            700 * u.nm,
        ]
    )
    def test__setitem__(
            self,
            array: na.ScalarArray,
            item: dict[str, int | slice | na.ScalarArray] | na.ScalarArray,
            value: float | na.ScalarArray
    ):
        super().test__setitem__(array=array, item=item, value=value)


class AbstractTestAbstractImplicitTemporalVectorArray(
    AbstractTestAbstractTemporalVectorArray,
    test_vectors_cartesian.AbstractTestAbstractImplicitCartesianVectorArray,
):
    pass


class AbstractTestAbstractParameterizedTemporalVectorArray(
    AbstractTestAbstractImplicitTemporalVectorArray,
    test_vectors_cartesian.AbstractTestAbstractParameterizedCartesianVectorArray,
):
    pass


class AbstractTestAbstractTemporalVectorSpace(
    AbstractTestAbstractParameterizedTemporalVectorArray,
    test_vectors_cartesian.AbstractTestAbstractParameterizedCartesianVectorArray,
):
    pass


def _temporal_linear_spaces() -> list[na.TemporalVectorLinearSpace]:
    return [
        na.TemporalVectorLinearSpace(1 * u.s, 2 * u.s, axis="y", num=_num_y)
    ]


@pytest.mark.parametrize("array", _temporal_linear_spaces())
class TestTemporalVectorlinearSpace(
    AbstractTestAbstractTemporalVectorSpace,
    test_vectors_cartesian.AbstractTestAbstractCartesianVectorLinearSpace,
):
    pass



from __future__ import annotations
from typing import Type, TypeVar
from typing_extensions import Self
import dataclasses
import named_arrays as na

__all__ = [
    'AbstractSpectralDirectionalVectorArray',
    'SpectralDirectionalVectorArray',
    'AbstractImplicitSpectralDirectionalVectorArray',
    'AbstractParameterizedSpectralDirectionalVectorArray',
    'AbstractSpectralDirectionalVectorSpace',
    'SpectralDirectionalVectorLinearSpace',
    'WcsSpectralDirectionalVectorArray',
]

DirectionT = TypeVar("DirectionT", bound=na.ArrayLike)
WavelengthT = TypeVar("WavelengthT", bound=na.ScalarLike)


@dataclasses.dataclass(eq=False, repr=False)
class AbstractSpectralDirectionalVectorArray(
    na.AbstractDirectionalVectorArray,
    na.AbstractSpectralVectorArray,
):

    @property
    def type_abstract(self) -> Type[na.AbstractArray]:
        return AbstractSpectralDirectionalVectorArray

    @property
    def type_explicit(self) -> Type[na.AbstractExplicitArray]:
        return SpectralDirectionalVectorArray

    @property
    def type_matrix(self) -> Type[na.SpectralDirectionalMatrixArray]:
        return na.SpectralDirectionalMatrixArray


@dataclasses.dataclass(eq=False, repr=False)
class SpectralDirectionalVectorArray(
    AbstractSpectralDirectionalVectorArray,
    na.DirectionalVectorArray[DirectionT],
    na.SpectralVectorArray[WavelengthT],
):

    @classmethod
    def from_scalar(
            cls: Type[Self],
            scalar: na.AbstractScalar,
            like: None | na.AbstractExplicitVectorArray = None,
    ) -> SpectralDirectionalVectorArray:
        return cls(wavelength=scalar, direction=scalar)


@dataclasses.dataclass(eq=False, repr=False)
class AbstractImplicitSpectralDirectionalVectorArray(
    AbstractSpectralDirectionalVectorArray,
    na.AbstractImplicitDirectionalVectorArray,
    na.AbstractImplicitSpectralVectorArray,
):
    pass


@dataclasses.dataclass(eq=False, repr=False)
class AbstractParameterizedSpectralDirectionalVectorArray(
    AbstractImplicitSpectralDirectionalVectorArray,
    na.AbstractParameterizedVectorArray,
):
    pass


@dataclasses.dataclass(eq=False, repr=False)
class AbstractSpectralDirectionalVectorSpace(
    AbstractParameterizedSpectralDirectionalVectorArray,
    na.AbstractVectorSpace,
):
    pass


@dataclasses.dataclass(eq=False, repr=False)
class SpectralDirectionalVectorLinearSpace(
    AbstractSpectralDirectionalVectorSpace,
    na.AbstractVectorLinearSpace,
):
    pass


@dataclasses.dataclass(eq=False, repr=False)
class WcsSpectralDirectionalVectorArray(
    AbstractImplicitSpectralDirectionalVectorArray,
    na.AbstractWcsVector,
):
    crval: AbstractSpectralDirectionalVectorArray = dataclasses.MISSING
    crpix: na.AbstractCartesianNdVectorArray = dataclasses.MISSING
    cdelt: AbstractSpectralDirectionalVectorArray = dataclasses.MISSING
    pc: na.AbstractSpectralDirectionalMatrixArray = dataclasses.MISSING
    shape_wcs: dict[str, int] = dataclasses.MISSING

    @property
    def _components_explicit(self) -> dict[str, na.ArrayLike]:
        return dict()



from __future__ import annotations
from typing import TypeVar, Generic, Type
import abc
import dataclasses
import named_arrays as na

__all__ = [
    'AbstractDirectionalMatrixArray',
    'DirectionalMatrixArray',
]

DirectionT = TypeVar('DirectionT', bound=na.AbstractVectorArray)


@dataclasses.dataclass(eq=False, repr=False)
class AbstractDirectionalMatrixArray(
    na.AbstractCartesianMatrixArray,
    na.AbstractDirectionalVectorArray,
):
    @property
    @abc.abstractmethod
    def direction(self) -> na.AbstractVectorArray:
        """
        The `direction` component of the matrix.
        """

    @property
    def type_abstract(self) -> Type[AbstractDirectionalMatrixArray]:
        return AbstractDirectionalMatrixArray

    @property
    def type_explicit(self) -> Type[DirectionalMatrixArray]:
        return DirectionalMatrixArray

    @property
    def type_vector(self) -> Type[na.DirectionalVectorArray]:
        return na.DirectionalVectorArray

    @property
    def determinant(self) -> na.ScalarLike:
        raise NotImplementedError


@dataclasses.dataclass(eq=False, repr=False)
class DirectionalMatrixArray(
    na.DirectionalVectorArray,
    AbstractDirectionalMatrixArray,
    na.AbstractExplicitMatrixArray,
    Generic[DirectionT],
):
    pass



from __future__ import annotations
from typing import Type

import dataclasses
import named_arrays as na

__all__ = [
    'AbstractSpectralDirectionalMatrixArray',
    'SpectralDirectionalMatrixArray',
]


@dataclasses.dataclass(eq=False, repr=False)
class AbstractSpectralDirectionalMatrixArray(
    na.AbstractSpectralDirectionalVectorArray,
):

    @property
    def type_abstract(self) -> Type[AbstractSpectralDirectionalMatrixArray]:
        return AbstractSpectralDirectionalMatrixArray

    @property
    def type_explicit(self) -> Type[SpectralDirectionalMatrixArray]:
        return SpectralDirectionalMatrixArray

    @property
    def type_vector(self) -> Type[na.SpectralDirectionalVectorArray]:
        return na.SpectralDirectionalVectorArray

    @property
    def determinant(self) -> na.ScalarLike:
        return NotImplementedError


@dataclasses.dataclass(eq=False, repr=False)
class SpectralDirectionalMatrixArray(
    na.SpectralDirectionalVectorArray,
    AbstractSpectralDirectionalMatrixArray,
    na.AbstractExplicitMatrixArray,
):
    pass



import numpy as np
import pytest
import astropy.units as u
import named_arrays as na
import named_arrays._vectors.tests.test_vectors

__all__ = [
    'AbstractTestAbstractMatrixArray',
    'AbstractTestAbstractExplicitMatrixArray',
    'AbstractTestAbstractImplicitMatrixArray',
    'AbstractTestAbstractMatrixRandomSample',
    'AbstractTestAbstractMatrixUniformRandomSample',
    'AbstractTestAbstractMatrixNormalRandomSample',
    'AbstractTestAbstractParameterizedMatrixArray',
]


class AbstractTestAbstractMatrixArray(
    named_arrays._vectors.tests.test_vectors.AbstractTestAbstractVectorArray,
):

    def test_rows(self, array: na.AbstractMatrixArray):
        assert np.all(array.rows == array.components)

    def test_is_consistent(self, array: na.AbstractMatrixArray):

        assert array.is_consistent

        array_2 = array.explicit.copy()
        array_2.x = 5
        assert not array_2.is_consistent

    def test_is_square(self, array: na.AbstractMatrixArray):
        assert array.is_square == (len(array.row_prototype.entries) == len(array.column_prototype.entries))

    def test_cartesian_nd(self, array: na.AbstractMatrixArray):
        cartesian_nd = array.cartesian_nd
        assert isinstance(cartesian_nd, na.AbstractCartesianNdMatrixArray)
        for c in cartesian_nd.components:
            component = cartesian_nd.components[c]
            assert isinstance(component, na.AbstractCartesianNdVectorArray)
            for c2 in component.components:
                assert isinstance(na.as_named_array(component.components[c2]), na.AbstractScalar)

    def test_from_cartesian_nd(self, array: na.AbstractMatrixArray):
        assert np.all(array.type_explicit.from_cartesian_nd(array.cartesian_nd, like=array) == array)

    def test_matrix_transpose(self, array: na.AbstractMatrixArray):
        result = array.matrix_transpose

        assert isinstance(result, array.components[next(iter(array.components))].type_matrix)
        assert all(isinstance(result.components[r], array.type_vector) for r in result.components)

        assert np.all(result.matrix_transpose == array)
        assert np.all(result + result == (array + array).matrix_transpose)
        if array.row_prototype.cartesian_nd.components.keys() == array.column_prototype.cartesian_nd.components.keys():
            assert np.all(result @ result == (array @ array).matrix_transpose)
        assert np.all(2 * result == (2 * array).matrix_transpose)
        if array.is_square:
            assert np.all(result.determinant == array.determinant)
            assert np.all(result.inverse == (array.inverse).matrix_transpose)

    def test_matrix(self, array: na.AbstractMatrixArray):
        assert np.all(array.matrix == array)

    def test_determinant(self, array: na.AbstractMatrixArray):
        if array.is_square:
            result = array.determinant

            assert isinstance(result, (int, float, complex, u.Quantity, na.AbstractScalar))

            assert np.all((2 ** 2) * result == (2 * array).determinant)
            assert np.all(result == array.matrix_transpose.determinant)
            assert np.allclose(result * result, (array @ array).determinant)
            assert np.allclose(result, 1 / array.inverse.determinant)

    def test_inverse(self, array: na.AbstractMatrixArray):
        if array.is_square:
            result = array.inverse

            assert isinstance(result, na.AbstractMatrixArray)

            identity = na.Cartesian2dMatrixArray(
                x=na.Cartesian2dVectorArray(1, 0),
                y=na.Cartesian2dVectorArray(0, 1),
            )

            assert np.allclose(array @ result, identity)
            assert np.allclose(result.inverse, array)
            assert np.allclose((2 * array).inverse, result / 2)

    class TestMatmul(
        named_arrays._vectors.tests.test_vectors.AbstractTestAbstractVectorArray.TestMatmul
    ):

        def test_matmul(
                self,
                array: float | u.Quantity | na.AbstractScalar | na.AbstractVectorArray | na.AbstractMatrixArray,
                array_2: float | u.Quantity | na.AbstractScalar | na.AbstractVectorArray | na.AbstractMatrixArray,
        ):
            try:
                assert np.all(array @ (array_2 + array_2) == array @ array_2 + array @ array_2)
                assert np.all((array + array) @ array_2 == array @ array_2 + array @ array_2)
                assert np.all(2 * (array @ array_2) == (2 * array) @ array_2)
                assert np.all((array @ array_2) * 2 == array @ (array_2 * 2))

            except TypeError:
                with pytest.raises(TypeError):
                    array @ array_2
                return

    class TestNamedArrayFunctions(
        named_arrays._vectors.tests.test_vectors.AbstractTestAbstractVectorArray.TestNamedArrayFunctions
    ):

        @pytest.mark.skip
        class TestPltPlotLikeFunctions(
            named_arrays._vectors.tests.test_vectors.AbstractTestAbstractVectorArray.TestNamedArrayFunctions.TestPltPlotLikeFunctions
        ):
            pass

        @pytest.mark.skip
        class TestOptimizeRoot(
            named_arrays.tests.test_core.AbstractTestAbstractArray.TestNamedArrayFunctions.TestOptimizeRoot,
        ):
            pass

        @pytest.mark.skip
        class TestOptimizeMinimum(
            named_arrays.tests.test_core.AbstractTestAbstractArray.TestNamedArrayFunctions.TestOptimizeMinimum,
        ):
            pass


class AbstractTestAbstractExplicitMatrixArray(
    AbstractTestAbstractMatrixArray,
    named_arrays._vectors.tests.test_vectors.AbstractTestAbstractExplicitVectorArray,
):
    pass


class AbstractTestAbstractExplicitMatrixArrayCreation(
    named_arrays._vectors.tests.test_vectors.AbstractTestAbstractExplicitVectorArrayCreation,
):
    pass


class AbstractTestAbstractImplicitMatrixArray(
    named_arrays._vectors.tests.test_vectors.AbstractTestAbstractImplicitVectorArray,
):
    pass


class AbstractTestAbstractMatrixRandomSample(
    AbstractTestAbstractImplicitMatrixArray,
    named_arrays._vectors.tests.test_vectors.AbstractTestAbstractVectorRandomSample,
):
    pass


class AbstractTestAbstractMatrixUniformRandomSample(
    AbstractTestAbstractMatrixRandomSample,
    named_arrays._vectors.tests.test_vectors.AbstractTestAbstractVectorUniformRandomSample,
):
    pass


class AbstractTestAbstractMatrixNormalRandomSample(
    AbstractTestAbstractMatrixRandomSample,
    named_arrays._vectors.tests.test_vectors.AbstractTestAbstractVectorNormalRandomSample,
):
    pass


class AbstractTestAbstractParameterizedMatrixArray(
    AbstractTestAbstractImplicitMatrixArray,
    named_arrays._vectors.tests.test_vectors.AbstractTestAbstractParameterizedVectorArray,
):
    pass
